/***************************************************************************
 *   Copyright (C) 2005 by Christian Muehlhaeuser, Last.fm Ltd.            *
 *   chris@last.fm                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include <QBuffer>
#include <QHttpResponseHeader>
#include <QTcpServer>
#include <QTcpSocket>

#include "ui_player.h"
#include "ui_playermini.h"

#include "webserviceconnector.h"
#include "playback.h"
#include "stationdialog.h"

class QMovie;

class CoverLoader;
class ImageFader;
class Song;

class Player : public QMainWindow
{
    Q_OBJECT

    public:
        Player( QWidget *parent = 0, QString cli = QString( "" ), QString dataPath = QString( "" ) );
        ~Player();

    protected:
        void closeEvent( QCloseEvent *event );

    private:
        Ui::PlayerForm ui;
        Ui::PlayerMiniForm ui_mini;

        WebserviceConnector *ws;
        StationDialog *station;
        Playback *playback;

        QTimer *m_timer;
        CoverLoader *coverLoader;
        QBuffer *coverBuffer;

        QTcpServer *serverSocket;
        QTcpSocket *webSocket;

        QUrl streamUrl;
        QString m_dataPath;

        QMenu *menu;
        QAction *stationAction;
        QAction *urlAction;
        QAction *recordAction;
        QAction *discoveryAction;
        QAction *refreshAction;
        QAction *configureAction;
        QAction *externalPlayerAction;
        QAction *onTopAction;
        QAction *aboutAction;

        ImageFader *fader;
        QMovie *animation;
        bool m_proxyConnected;

        void loadDefaultCover();

        void dragEnterEvent( QDragEnterEvent *event );
        void dropEvent( QDropEvent *event );

    public slots:
        void handshake();

        void skipSong();
        void loveSong();
        void banSong();

        void recordToProfile( bool checked );
        void discoveryMode( bool checked );
        void setStayOnTop( bool checked );
        void useExternalPlayer( bool checked );

    private slots:
        void init();
        void switchMode();

        void openAbout();
        void openConfig();
        void openMenu();
        void openStationDialog();
        void openUrlDialog();
        void openTagDialog();
        void openJournalMenu();

        void webConnect();
        void webRequest();

        void handshakeResult( bool connected, bool loginDone, const QUrl& streamUrl );
        void metaDataResult( const Song& song, bool discovery );
        void changeStationPrepared();
        void changeStationResult( bool error );

        void skipResult();
        void banResult();

        void animationStart();
        void animationStop();
        void coverLoaded( bool error );
        void updateTimeBar();

        void playbackStarting();
        void playbackFinished();
        void volumeChanged( int value );
        void stopPlayback();

        void browseArtist();
        void browseAlbum();
        void browseTrack();

        void journalArtist();
        void journalAlbum();
        void journalTrack();

        void proxyClientConnected();
        void proxyClientDisconnected();
};
