/***************************************************************************
 *   Copyright (C) 2005 by Christian Muehlhaeuser, Last.fm Ltd.            *
 *   chris@last.fm                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include <QtCore>

#include "ui_settings.h"

class BrowserThread : public QThread
{
    public:
        void run();
};

class Settings : public QDialog
{
    Q_OBJECT

    static Settings *s_instance;

    public:
        Settings( QWidget *parent = 0 );
        static Settings *instance() { return s_instance; }

        QString dataPath();
        void setDataPath( QString path );

        void setInitial( bool initial );

        bool debug();
        void setDebug( bool debug );

        bool actAsProxy();
        void setActAsProxy( bool proxy );

        int volume();
        void setVolume( int volume );

        int positionX();
        void setPositionX( int x );

        int positionY();
        void setPositionY( int y );

        int soundSystem();
        void setSoundSystem( int system );

        int soundCard();
        void setSoundCard( int card );

        QString username();
        void setUsername( QString username );

        QString password();
        void setPassword( QString password );

        QString browser();
        void setBrowser( QString browser );

        int bufferSize();
        void setBufferSize( int value );

        QString proxyUsername();
        void setProxyUsername( QString username );

        QString proxyPassword();
        void setProxyPassword( QString password );

        QString proxyHost();
        void setProxyHost( QString host );

        int proxyPort();
        void setProxyPort( int port );

        bool proxyUsage();
        void setProxyUsage( bool enabled );

        int stationCounter( QString url );
        QString stationName( QString url );

        void incrementStationCounter( QString url, QString stationName = QString::null );
        void deleteStationCounter( QString url );
        void resetStationCounter( QString url );
        void clearStationCounters();

        QStringList stationItems();

        void startBrowser( QString url );

    public slots:
        void okPressed();
        void signUp();

    signals:
        void configChanged();

    private:
        Ui::SettingsForm ui;

        QSettings *config;

        QString m_dataPath;

        bool m_debug;
        bool m_actAsProxy;
        bool m_initial;

        QString originalUsername;
        QString originalPassword;
        QString originalProxy;
};

