/***************************************************************************
 *   Copyright (C) 2005 by Christian Muehlhaeuser, Last.fm Ltd.            *
 *   chris@last.fm                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifdef WIN32
#include <windows.h>
#endif

#include <QtGui>
#include <QTimer>

#include "settings.h"
#include "song.h"
#include "tagdialog.h"
#include "webserviceconnector.h"


TagDialog::TagDialog( QWidget *parent, Song &song, WebserviceConnector *webserviceConnector )
    : QDialog( parent, Qt::Dialog )
    , ws( webserviceConnector )
{
    ui.setupUi( this );

    m_song = new Song();
    m_song->fetch( song );

    connect( ws, SIGNAL( tagsForArtistResult( QStringList ) ), this, SLOT( tagsForArtistResult( QStringList ) ) );
    connect( ws, SIGNAL( tagsForUserResult( QStringList ) ), this, SLOT( tagsForUserResult( QStringList ) ) );
    connect( ws, SIGNAL( tagsForUserArtistResult( QStringList ) ), this, SLOT( tagsForUserArtistResult( QStringList ) ) );
    connect( ws, SIGNAL( tagsForUserTrackResult( QStringList ) ), this, SLOT( tagsForUserTrackResult( QStringList ) ) );
    connect( ws, SIGNAL( tagsForUserAlbumResult( QStringList ) ), this, SLOT( tagsForUserAlbumResult( QStringList ) ) );

    connect( ui.tagTypeBox, SIGNAL( activated( int ) ), this, SLOT( tagTypeChanged( int ) ) );
    connect( ui.okButton, SIGNAL( clicked() ), this, SLOT( okPressed() ) );
    connect( ui.collapseButton, SIGNAL( clicked() ), this, SLOT( toggleCollapse() ) );
    connect( ui.personalTagsList, SIGNAL( itemDoubleClicked( QListWidgetItem* ) ), this, SLOT( addTag( QListWidgetItem* ) ) );
    connect( ui.publicTagsList, SIGNAL( itemDoubleClicked( QListWidgetItem* ) ), this, SLOT( addTag( QListWidgetItem* ) ) );

    ws->tagsForArtist( m_song->artist() );
    ws->tagsForUser( Settings::instance()->username() );

    tagTypeChanged( 0 );

}


int
TagDialog::execSmall()
{
    setModal( true );
    show();

    QApplication::processEvents();

    delete layout();
    collapsed = false;
    toggleCollapse();

    while ( isVisible() )
    {
        #ifdef WIN32
        Sleep( 10 );
        #endif
        #ifndef WIN32
        usleep( 10 );
        #endif

        QApplication::processEvents();
    }

    done( result() );
    return result();
}


void
TagDialog::okPressed()
{
    ws->setTag( m_song, ui.tagTypeBox->currentIndex(), ui.tagEdit->text() );
}


void
TagDialog::toggleCollapse()
{
    if ( collapsed )
    {
        collapsed = false;

        QIcon image( Settings::instance()->dataPath() + "data/collapse.png" );
        ui.collapseButton->setIcon( image );
        ui.collapseLabel->setText( "Hide favourites" );

        resize( 393, 373 );
        setMinimumSize( 393, 373 );
        setMaximumSize( 393, 373 );

        setSizeGripEnabled( false );
    }
    else
    {
        collapsed = true;

        QIcon image( Settings::instance()->dataPath() + "data/expand.png" );
        ui.collapseButton->setIcon( image );
        ui.collapseLabel->setText( "Show favourites" );

        resize( 393, ui.extraLabel1->y() - 4 );
        setMinimumSize( 393, ui.extraLabel1->y() - 4 );
        setMaximumSize( 393, ui.extraLabel1->y() - 4 );
    }
}


void
TagDialog::tagTypeChanged( int type )
{
    ui.tagTypeBox->setEnabled( false );
//     ui.tagEdit->setEnabled( false );
    ui.tagEdit->setText( "Loading..." );

    switch ( type )
    {
        case 0:
            ws->tagsForUserArtist( Settings::instance()->username(), m_song->artist() );
            break;

        case 1:
            ws->tagsForUserAlbum( Settings::instance()->username(), m_song->artist(), m_song->album() );
            break;

        case 2:
            ws->tagsForUserTrack( Settings::instance()->username(), m_song->artist(), m_song->track() );
            break;
    }
}


void
TagDialog::tagsForArtistResult( const QStringList &result )
{
    for ( int i = 0; i < result.count(); i++ )
    {
        ui.publicTagsList->addItem( result[i] );
    }
}


void
TagDialog::tagsForUserResult( const QStringList &result )
{
    for ( int i = 0; i < result.count(); i++ )
    {
        ui.personalTagsList->addItem( result[i] );
    }
}


void
TagDialog::tagsForUserArtistResult( const QStringList &result )
{
    QString tags = "";

    for ( int i = 0; i < result.count(); i++ )
    {
        tags += ( i > 0 ? ", " : "" ) + result[i];
    }

    ui.tagTypeBox->setEnabled( true );
    ui.tagEdit->setEnabled( true );
    ui.tagEdit->setText( tags );
    ui.tagEdit->setFocus();
}


void
TagDialog::tagsForUserTrackResult( const QStringList &result )
{
    QString tags = "";

    for ( int i = 0; i < result.count(); i++ )
    {
        tags += ( i > 0 ? ", " : "" ) + result[i];
    }

    ui.tagTypeBox->setEnabled( true );
    ui.tagEdit->setEnabled( true );
    ui.tagEdit->setText( tags );
    ui.tagEdit->setFocus();
}


void
TagDialog::tagsForUserAlbumResult( const QStringList &result )
{
    QString tags = "";

    for ( int i = 0; i < result.count(); i++ )
    {
        tags += ( i > 0 ? ", " : "" ) + result[i];
    }

    ui.tagTypeBox->setEnabled( true );
    ui.tagEdit->setEnabled( true );
    ui.tagEdit->setText( tags );
    ui.tagEdit->setFocus();
}


void
TagDialog::addTag( QListWidgetItem *item )
{
    if ( !ui.tagEdit->text().trimmed().isEmpty() )
        ui.tagEdit->setText( ui.tagEdit->text() + ", " + item->text() );
    else
        ui.tagEdit->setText( item->text() );
}

